% Find analytic solution of homogeneous ODE with real eigenvalues:
% y^n + a_(n-1)*y^(n-1) + ... + a1*y' +a0*y = 0
% general solution:
% y = C1*e^(r1*t) + C2*e^(r2*t) + C3*e^(r3*t) + C4*e^(r4*t)

clc;
clear;
close all



% Example:
% y'''' -5y''' -7y'' +29y' +30y = 0

% solve characteristic equation
% r^4 - 5*r^3 - 7*r^2 + 29*r + 30 = 0

% find roots 
% a = [a0 a1 a2 ... an]
a=[30 29 -7 -5 1]'; % analytic solution of ODE
% a=[0 9 0 -120 0 432 0 -576 0 256]'; % example in presentation


x=roots(a(end:-1:1)); % solution in MATLAB
%     5.0000  3.0000  -2.0000  -1.0000

x=sort(x,'descend')

%%% Analytic solution of initial value problem
% y'''' -5y''' -7y'' +29y' +30y = 0
% y(0)=1, y'(0)=2, y''(0)=-2, y'''(0)=-4
% init=[-4;-2;2;1];
init=[1;2;-2;-4];
% real different eigenvalues
% r1=5, r2=3, r3=-1, r4=-2
% General form
% y=C1*e^(r1*t)+C2*e^(r2*t)+C3*e^(r3*t)+C4*e^(r4*t)
% Higher derivatives
% y'=C1*r1*e^(r1*t)+C2*r2*e^(r2*t)+C3*r3*e^(r3*t)+C4*r4*e^(r4*t)
% y''=C1*(r1^2)*e^(r1*t)+C2*(r2^2)*e^(r2*t)+C3*(r3^2)*e^(r3*t)+C4*(r4^2)*e^(r4*t)
% y'''=C1*(r1^3)*e^(r1*t)+C2*(r2^3)*e^(r2*t)+C3*(r3^3)*e^(r3*t)+C4*(r4^3)*e^(r4*t)

% integration constants C1,...C4
C=[ 1    1    1    1
    x(1) x(2) x(3) x(4);
    x(1)^2 x(2)^2 x(3)^2 x(4)^2;
    x(1)^3 x(2)^3 x(3)^3 x(4)^3;]\init;


y_anal=@(t) C(1)*exp(x(1)*t)+C(2)*exp(x(2)*t)+C(3)*exp(x(3)*t)+C(4)*exp(x(4)*t);

% Method of derivative order reduction
% y''''= 5y''' +7y'' -29y' -30y 
% p3y'=5*p3y +7*p2y -29*py -30*y, y'''(0)=-4
% p2y'=p3y, y''(0)=-2
% py'=p2y, y'(0)=2
% y'=py, y(0)=1

init=[-4;-2;2;1];
A=[5 7 -29 -30;
   1 0 0 0;
   0 1 0 0;
   0 0 1 0];


myodefun = @(t,y) A*y;

tspan=[0,1];
tol=1e-5;
ABSTOL(1:length(init))=tol;
options = odeset('RelTol',tol,'AbsTol',ABSTOL);%[1e-10 1e-10 1e-10 1e-10]

[T,Y] = ode23(myodefun,tspan,init, options);

figure
hold on
plot(T,Y(:,4))
plot(T,y_anal(T),'--','LineWidth',2)
hold off
legend('y','y_{anal}')
xlabel('t')
