function simulate_SLAE
clc
clear
global A b A_pos b_pos
% syms lambda A_lambda
% load ab.mat



% example from lecture
% A=[1 1;1 -1]
% b=[1;3]

%%% h(A)...rank of square matrix A
%%% h(A)...rank of rectangle matrix [A,b]
%%% matrix A is regular, h(A)=h(A|b), A is positive definite
% A = [6 3 2;
%     1 8 3;
%     2 1 9]
% b = [1;2;3]


%%% matrix A is regular, h(A)=h(A|b), A is indefinite
% A = [-6 -3 -2;
%     1 8 3;
%     2 1 9]
% b = [-1;2;3]



%%% matrix A is singular, h(A)<h(A|b) no solution
% A = [6 3 2;
%     1 8 3;
%     12 6 4]
% b = [1;2;3]

%%% matrix A is singular, h(A)=h(A|b)<number of equations, infinite number of solutions
A = [6 3 2;
    1 8 3;
    12 6 4]
b = [1;2;2]




fprintf('\n det(A): %g \n ', det(A));
fprintf('\n rank(A): %g rank(A|b): %g \n ', rank(A), rank([A,b]));
fprintf('\n max(eig(A)): %g \n min(eig(A)): %g \n ', max(eig(A)), min(eig(A)));


x_backslash=A\b;

x_inv=inv(A)*b;

x_linsolve=linsolve(A,b);


fprintf('||x_backslash-x_linsolve||: %g\n ', norm(x_backslash-x_linsolve));


%%% A_pos...positive definite matrix
A_pos=A'*A;
b_pos=A'*b;

% rank(A_pos);
fprintf('\n max(eig(A_pos)): %g \n min(eig(A_pos)): %g \n ', max(eig(A_pos)), min(eig(A_pos)));

%%% let's try various initial conditions
% y0=zeros(size(b,1),1);
 y0=10*ones(size(b,1),1);
tmax=50;

odeSolve=ode15s(@SLAE_At,[0,tmax],y0); % Solution with 'Stable transformation' A_pos=(A^T*A)
% odeSolve=ode15s(@SLAE_A,[0,tmax],y0);


figure
plot(odeSolve.x, odeSolve.y)
legend('x_1','x_2','x_3')

% numerical solution of SLAE using ODEs
x_odeSolution=odeSolve.y(:,end)

% x_odeSolution_prev=(inv(A')*A_pos)\(inv(A')*b_pos);
% x_odeSolution_prev=A'\x_odeSolution;
%%% Error
fprintf('||x_odeSolution-x_linsolve||: %g\n ', norm(x_odeSolution-x_linsolve));

%%% residuum
res_odeSolution=-A*x_odeSolution+b
% res_odeSolution=norm(-A*x_odeSolution+b)
end




function dydt = SLAE_At(t, y)
%% Solution with 'Stable transformation' A_pos=(A^T*A)
global A_pos b_pos;
dydt = -A_pos * y + b_pos;

end


function dydt = SLAE_A(t, y)
%% Solution with matrix A
global A b;
dydt = -A * y + b;

end

